assert(TrackCooldowns, "TrackCooldowns not found!")

TC = {
	VersionBuild = "2",
	VersionMinor = "3",
	VersionMajor = "2",
}

local AceConfig = LibStub("AceConfigDialog-3.0")
local Media = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TrackCooldowns")

local TrackCooldowns = TrackCooldowns
local mod = TrackCooldowns:NewModule("Display", nil, "AceComm-3.0", "AceEvent-3.0", "AceConsole-3.0", "LibBars-1.0")

local pairs = _G.pairs
local UnitName, UnitClass = _G.UnitName, _G.UnitClass
local tonumber, tostring, type = _G.tonumber, _G.tostring, _G.type

local barGroup = nil
local function sortFunc(a, b)
	if a.isTimer ~= b.isTimer then
		return a.isTimer
	end
	
	if a.value == b. value then
		return a.name > b.name
	else
		return a.value > b.value
	end
end

local playerName

local defaults = {
	profile = {
		enabled				= true,

		locked				= false,
		scale				= 100.0,
		texture				= "Aluminium",
		growUp				= true,
		clamped				= true,
		fontFace			= nil,
		fontSize			= 12,
		orientation			= 3,
		alpha				= 100.0,
		
		width				= 180,
		height				= 16,
		
		showOwn				= false,
		hideSelf			= false,
		spells				= { },
		
		inRaids				= true,
		inParties			= true,
		inBattlegrounds		= true,
		
		readyMessage		= true,
		startMessage		= true,
		reportChannel		= "SELF",
		useCustomChannel	= false,
		customChannel		= "",

		casterName			= true,
		spellName			= true,
		tooltip				= true,
	}
}

local optFrame

function mod:OnInitialize()
end

function mod:OnEnable()
	self.db = LibStub("AceDB-3.0"):New("TC_DB", nil, "Default")
	
	if not self.db.profile.spells or #self.db.profile.spells == 0 then
		for k,v in pairs(TrackCooldowns.cooldowns) do
			for k2, v2 in pairs(v) do
				defaults.profile.spells[v2.id] = true
			end
		end
	end
	
	self.db:RegisterDefaults(defaults)
	optFrame = AceConfig:AddToBlizOptions(L["Track Cooldowns"], L["Track Cooldowns"])
	playerName = UnitName("player")
	
	self:CreateFrame()
	self:SetAnchors(true)
	self:UpdateDisplay()
end

function mod:OnDisable()
end

function mod:StartCooldown(sender, spellId, cooldown)
	if self.db.profile.showOwn and sender ~= playerName then return end
	if self.db.profile.hideSelf and sender == playerName then return end
	if not self.db.profile.spells[spellId] then return end
	if not self.db.profile.enabled then	return end

	local bar = barGroup:GetBar(sender .. "_" .. spellId)
	local spellTitle

	if self.db.profile.casterName and self.db.profile.spellName then
		spellTitle = sender .. ": " .. GetSpellInfo(spellId)
	elseif self.db.profile.casterName then
		spellTitle = sender
	elseif self.db.profile.spellName then
		spellTitle = GetSpellInfo(spellId)
	else
		spellTitle = ""
	end

	if bar == nil then
		bar = barGroup:NewTimerBar((sender .. "_" .. spellId), spellTitle, cooldown, cooldown, spellId)
		bar.caster  = sender
		bar.spellId = spellId
		bar.spellName = GetSpellInfo(spellId)

		if self.db.profile.tooltip then
			bar:SetScript("OnEnter", function(bar, ...) 
				GameTooltip:SetOwner(bar, "ANCHOR_TOPRIGHT")
				GameTooltip:SetText(bar.caster .. ": " .. bar.spellName)
				GameTooltip:Show()
			end)
			bar:SetScript("OnLeave", function(frame, ...)
				GameTooltip:Hide()
			end)
			bar:EnableMouse(true)
		else
			bar:EnableMouse(false)
		end

		if self.db.profile.startMessage then
			local spellLink
			spellLink = GetSpellLink(spellId)
			if spellLink then
				if self.db.profile.useCustomChannel and self.db.profile.customChannel ~= "" then
					SendChatMessage(L["Track Cooldowns"] .. ": " .. L["%s has casted %s."]:format(sender, spellLink), "CHANNEL", nil, self.db.profile.customChannel)
				elseif self.db.profile.reportChannel == "SELF" then
					TrackCooldowns:Print(L["%s has casted %s."]:format("|cff66ccff" ..  sender .. "|r", spellLink))
				else
					SendChatMessage(L["Track Cooldowns"] .. ": " .. L["%s has casted %s."]:format(sender, spellLink), self.db.profile.reportChannel)
				end
			end
		end

	else
		bar:SetTimer(cooldown, bar.maxValue)
	end
	
	local _, c = UnitClass(sender)
	if not c then
		for k, v in pairs(TrackCooldowns.cooldowns) do
			for k2, v2 in pairs(v) do
				if v2.id == spellId then
					c = k
				end
			end
		end
	end
	
	local color = RAID_CLASS_COLORS[c]

	if type(color) == "table" then
		bar:SetColorAt(1.00, color.r, color.g, color.b, 1)
		bar:SetColorAt(0.00, color.r, color.g, color.b, 1)
	end

end

function mod:StopCooldown(sender, spellId)
	local bar = barGroup:GetBar(sender .. "_" .. spellId)

	if bar ~= nil then
		barGroup:RemoveBar(bar)
		self:UpdateDisplay()
	end

end

function mod:PLAYER_LOGIN()
end

--[[		Configuration Methods		]]--

function mod:TrackCooldownsToggle()
	self = mod

	if self.db.profile.enabled then
		self.db.profile.enabled = false
		AceConfig:Close(L["Track Cooldowns"], configFrame)
		TrackCooldowns:Print(L["is disabled."])
	else
		self.db.profile.enabled = true
		AceConfig:Close(L["Track Cooldowns"], configFrame)
		TrackCooldowns:Print(L["is enabled."])
	end

end

function mod:ShowVersion()
	TrackCooldowns:Print(L["Version"] .. " - |cff66ccff" .. TC.VersionMajor .. "." .. TC.VersionMinor .. "." .. TC.VersionBuild .. "|r")
end

function mod:ShowConfig()
	AceConfig:SetDefaultSize(L["Track Cooldowns"], 500, 550)
	AceConfig:Open(L["Track Cooldowns"], configFrame)
end

function mod:Test()
	self = mod
	local i = 1
	for _, v in pairs(TrackCooldowns.cooldowns) do
		for _, v2 in pairs(v) do
			self:StartCooldown(L["Test"], v2.id, i * 2)
			i = i + 1
		end
	end
end

function mod:Reset()
	local bars = barGroup:GetBars()

	if type(bars) == "table" then
		for k, v in pairs(bars) do
			barGroup:RemoveBar(k)
		end
	end

end

function mod:SpellToggle(spell, val)
	if val == true then return end
	local bars = barGroup:GetBars()
	local removed = false

	if type(bars) == "table" then
		for k, v in pairs(bars) do
			if spell == v.spellId then
				barGroup:RemoveBar(k)
				removed = true
			end
		end
	end

	if removed then
		self:UpdateDisplay()
	end
end

--[[		Bar Group Anchor		]]--

function mod:CreateFrame()
	barGroup = nil
	barGroup = self:NewBarGroup(L["Track Cooldowns"], nil, self.db.profile.width, self.db.profile.height, "TCD_Anchor")
	barGroup:SetFlashPeriod(0)
	barGroup:SetSortFunction(sortFunc)
	barGroup.RegisterCallback(self, "AnchorClicked")
	barGroup.RegisterCallback(self, "AnchorMoved")
	barGroup.RegisterCallback(self, "TimerFinished")
end

function mod:UpdateDisplay()
	if not barGroup then return end
	
	if self.db.profile.locked then
		barGroup:Lock()
		barGroup:HideAnchor()
	else
		barGroup:Unlock()
		barGroup:ShowAnchor()
	end

	barGroup:SetOrientation(self.db.profile.orientation)
	barGroup:SetClampedToScreen(self.db.profile.clamped)
	barGroup:SetFont(Media:Fetch("font", self.db.profile.fontFace), self.db.profile.fontSize)
	barGroup:SetTexture(Media:Fetch("statusbar", self.db.profile.texture))
	barGroup:SetScale(self.db.profile.scale / 100.0)
	barGroup:ReverseGrowth(self.db.profile.growUp)
	barGroup:SetAlpha(self.db.profile.alpha / 100.0)
	barGroup:SetWidth(self.db.profile.width)
	barGroup:SetHeight(self.db.profile.height)
end

function mod:SetAnchors(useDB)
	local t = self.db.profile.growUp
	local x, y

	if useDB then
		x, y = self.db.profile.posX, self.db.profile.posY
		if not x and not y then
			barGroup:ClearAllPoints()
			barGroup:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
			return
		end
	elseif t then
		x, y = barGroup:GetLeft(), barGroup:GetBottom()
	else
		x, y = barGroup:GetLeft(), barGroup:GetTop()
	end

	barGroup:ClearAllPoints()

	if t then
		barGroup:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", x, y)
	else
		barGroup:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
	end

	self.db.profile.posX, self.db.profile.posY = x, y
end

function mod:AnchorClicked(cbk, group, button)
	if button == "RightButton" then
		self:ShowConfig()
	end
end

function mod:AnchorMoved(cbk, group, x, y)
	self:SetAnchors()
end

function mod:TimerFinished(cbk, group, bar, name)
	if not self.db.profile.readyMessage then return end
	local spellLink
	spellLink = GetSpellLink(bar.spellId)
	if spellLink then
		if self.db.profile.useCustomChannel and self.db.profile.customChannel ~= "" then
			SendChatMessage(L["Track Cooldowns"] .. ": " .. L["%s's %s is up."]:format(bar.caster, spellLink), "CHANNEL", nil, self.db.profile.customChannel)
		elseif self.db.profile.reportChannel == "SELF" then
			TrackCooldowns:Print(L["%s's %s is up."]:format("|cff66ccff" ..  bar.caster .. "|r", spellLink))
		else
			SendChatMessage(L["Track Cooldowns"] .. ": " .. L["%s's %s is up."]:format(bar.caster, spellLink), self.db.profile.reportChannel)
		end
	end
end